-- a spinner control - textbox with enumerated values and up/down buttons for selecting
local importPath=string.gsub(getfenv(1)._.Name,"%.Spinner","").."."
local resourcePath=string.gsub(importPath,"%.","/").."Resources/"

-- requires fontMetric
if FontMetric==nil then
	import (importPath.."FontSupport")
	fontMetric=FontMetric()
end

Spinner = class( Turbine.UI.Control )

function Spinner:Constructor()
	self.Font=Turbine.UI.Lotro.Font.Verdana20
	if Settings~=nil and Settings.Font~=nil then self.Font=Settings.Font end
	local size=Turbine.UI.Lotro.FontInfo[self.Font].size
	Turbine.UI.Control.Constructor( self )
	self.TextBox=Turbine.UI.TextBox()
	self.TextBox:SetParent(self)
	self.TextBox:SetPosition(0,0)
	self.TextBox:SetFont(self.Font)
	self.TextBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
	self.TextBox:SetMultiline(false)
	fontMetric:SetFont(self.Font)
	self.TextBox:SetSize(fontMetric:GetTextWidth("8"),size) -- default size is a single digit
	self.Values={}-- enumerated table of label/value pairs	
	self.WrapAround=true -- default behavior is to allow wrapping when reaching end of values
	self.currentIndex=0
	self.TextBox:SetText("")
	self.TextBox:SetEnabled(false)
	self.TextBox.TextChanged=function()
		-- try to match the text to a value
	end

	self.Up=Turbine.UI.Control()
	self.Up:SetParent(self)
	self.Up:SetSize(10,10)
	self.Up:SetPosition(self.TextBox:GetLeft()+self.TextBox:GetWidth(),0)
	self.Up:SetBackground(0x4100028c) -- regular 0x4100028c, highlight 0x4100028d
	self.Up:SetBlendMode(Turbine.UI.BlendMode.Overlay)
	self.Up.MouseDown=function()
		self.Up:SetBackground(0x4100028d)
	end
	self.Up.MouseUp=function()
		self.Up:SetBackground(0x4100028c)
	end
	self.Up.MouseClick=function()
		local newIndex=self.currentIndex-1
		if newIndex==0 then
			if self.WrapAround then
				newIndex=#self.Values
			else
				newIndex=1
			end
		end
		self.currentIndex=newIndex
		self.TextBox:SetText(self.Values[self.currentIndex])
		if self.SelectedIndexChanged~=nil then self.SelectedIndexChanged() end
	end
	self.Down=Turbine.UI.Control()
	self.Down:SetParent(self)
	self.Down:SetSize(10,10)
	self.Down:SetPosition(self.TextBox:GetLeft()+self.TextBox:GetWidth(),10)
	self.Down:SetBackground(0x4100028e) -- regular 0x4100028e, highlight 0x4100028f
	self.Down:SetBlendMode(Turbine.UI.BlendMode.Overlay)
	self.Down.MouseDown=function()
		self.Down:SetBackground(0x4100028f)
	end
	self.Down.MouseUp=function()
		self.Down:SetBackground(0x4100028e)
	end
	self.Down.MouseClick=function()
		local newIndex=self.currentIndex+1
		if newIndex>#self.Values then
			if self.WrapAround then
				newIndex=1
			else
				newIndex=#self.Values
			end
		end
		self.currentIndex=newIndex
		self.TextBox:SetText(self.Values[self.currentIndex])
		if self.SelectedIndexChanged~=nil then self.SelectedIndexChanged() end
	end
end
function Spinner:SetFont(font)
	local size=Turbine.UI.Lotro.FontInfo[font].size
	self.Font=font
	fontMetric:SetFont(font)
	if size<20 then size=20 end -- minimum height 20 due to buttons
	self.TextBox:SetFont(font)
	local width=5
	for k,v in ipairs(self.Values) do
		local tmpWidth=fontMetric:GetTextWidth(tostring(v))
		if tmpWidth>width then width=tmpWidth end
	end
	self.TextBox:SetSize(width,size)
	local top=math.floor((size-20)/2)
	self.Up:SetPosition(self.TextBox:GetLeft()+self.TextBox:GetWidth(),top)
	self.Down:SetPosition(self.TextBox:GetLeft()+self.TextBox:GetWidth(),self.Up:GetTop()+self.Up:GetHeight())
	self:SetSize(self.Up:GetLeft()+self.Up:GetWidth(),size)
end
function Spinner:SetValues(valueList)
	local currentValue=self:GetValue()
	self.Values={}
	local newIndex
	for k,v in ipairs(valueList) do
		if newIndex==nil then newIndex=k end
		self.Values[k]=v
		if v==currentValue then
			newIndex=k
		end
	end
	self:SetFont(self.Font) -- will refresh display
	self:SetSelectedIndex(newIndex)
end
function Spinner:SetSelectedValue(value)
	-- will attempt to find the specified value in the internal values list and select it
	local newIndex
	for k,v in ipairs(self.Values) do
		if v==value then newIndex=k end
	end
	if newIndex~=nil then self:SetSelectedIndex(newIndex) end
end
function Spinner:AddValue(value, index)
	if index==nil then index=#self.Values+1 end
	table.insert(self.Values,index,value)
	self:SetFont(self.Font) -- will refresh display
end
function Spinner:RemoveValue(index)
	index=tonumber(index)
	if index==nil or index<=#self.Values then	
		table.remove(self.Values,index)
	end
	if index==currentIndex then
		if index>#self.Values then index=#self.Values end
		self:SetSelectedIndex(index)
	end
	self:SetFont(self.Font) -- will refresh display
end
function Spinner:SetSelectedIndex(newIndex)
	newIndex=tonumber(newIndex)
	if newIndex~=nil then
		if newIndex==0 then
			self.currentIndex=0
			self.TextBox:SetText("")
		elseif newIndex<=#self.Values then
			self.currentIndex=newIndex
			self.TextBox:SetText(self.Values[newIndex])
		end
	end
end
function Spinner:GetSelectedIndex()
	return self.currentIndex
end
function Spinner:GetValue()
	return self.Values[self.currentIndex]
end
function Spinner:SetWrapAround(value)
	if value==nil then value=true end -- "SetWrapAround" implies default is to set it true
	Spinner.WrapAround=(value~=false)
end
function Spinner:SetEntryAllowed(value)
	-- note, allowing user entry can be VERY bad...
	self.EntryAllowed=value
	self.TextBox:SetEnabled(value)
end
function Spinner:ClearValues()
	self.Values={}
	self.currentIndex=0
	self:SetFont(self.Font) -- will refresh display
end